// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_BACKGROUND_SYNC_SERVICE_MOJOM_H_
#define CONTENT_COMMON_BACKGROUND_SYNC_SERVICE_MOJOM_H_

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_impl.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "content/common/background_sync_service.mojom-internal.h"
#include "content/public/common/background_sync.mojom.h"
#include "content/public/common/permission_status.mojom.h"
namespace content {

enum BackgroundSyncError {
  BACKGROUND_SYNC_ERROR_NONE,
  BACKGROUND_SYNC_ERROR_STORAGE,
  BACKGROUND_SYNC_ERROR_NOT_FOUND,
  BACKGROUND_SYNC_ERROR_NO_SERVICE_WORKER,
  BACKGROUND_SYNC_ERROR_MAX = BACKGROUND_SYNC_ERROR_NO_SERVICE_WORKER,
};

class BackgroundSyncService;
using BackgroundSyncServicePtr = mojo::InterfacePtr<BackgroundSyncService>;

class BackgoundSyncServiceClient;
using BackgoundSyncServiceClientPtr = mojo::InterfacePtr<BackgoundSyncServiceClient>;


class BackgroundSyncServiceProxy;
class BackgroundSyncServiceStub;

class BackgroundSyncServiceRequestValidator;
class BackgroundSyncServiceResponseValidator;

class BackgroundSyncService {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = BackgroundSyncServiceProxy;
  using Stub_ = BackgroundSyncServiceStub;

  using RequestValidator_ = BackgroundSyncServiceRequestValidator;
  using ResponseValidator_ = BackgroundSyncServiceResponseValidator;
  virtual ~BackgroundSyncService() {}

  using RegisterCallback = mojo::Callback<void(BackgroundSyncError, content::SyncRegistrationPtr)>;
  virtual void Register(content::SyncRegistrationPtr options, int64_t service_worker_registration_id, const RegisterCallback& callback) = 0;

  using GetRegistrationCallback = mojo::Callback<void(BackgroundSyncError, content::SyncRegistrationPtr)>;
  virtual void GetRegistration(content::BackgroundSyncPeriodicity periodicity, const mojo::String& tag, int64_t service_worker_registration_id, const GetRegistrationCallback& callback) = 0;

  using GetRegistrationsCallback = mojo::Callback<void(BackgroundSyncError, mojo::Array<content::SyncRegistrationPtr>)>;
  virtual void GetRegistrations(content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetRegistrationsCallback& callback) = 0;

  using UnregisterCallback = mojo::Callback<void(BackgroundSyncError)>;
  virtual void Unregister(content::BackgroundSyncPeriodicity periodicity, int64_t id, const mojo::String& tag, int64_t service_worker_registration_id, const UnregisterCallback& callback) = 0;

  using GetPermissionStatusCallback = mojo::Callback<void(BackgroundSyncError, content::PermissionStatus)>;
  virtual void GetPermissionStatus(content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetPermissionStatusCallback& callback) = 0;
};


class BackgoundSyncServiceClientProxy;
class BackgoundSyncServiceClientStub;

class BackgoundSyncServiceClientRequestValidator;

class BackgoundSyncServiceClient {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = BackgoundSyncServiceClientProxy;
  using Stub_ = BackgoundSyncServiceClientStub;

  using RequestValidator_ = BackgoundSyncServiceClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  virtual ~BackgoundSyncServiceClient() {}

  virtual void Sync(content::SyncRegistrationPtr event) = 0;
};


class BackgroundSyncServiceProxy
    : public BackgroundSyncService,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit BackgroundSyncServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void Register(
      content::SyncRegistrationPtr options, int64_t service_worker_registration_id, const RegisterCallback& callback
  ) override;
  void GetRegistration(
      content::BackgroundSyncPeriodicity periodicity, const mojo::String& tag, int64_t service_worker_registration_id, const GetRegistrationCallback& callback
  ) override;
  void GetRegistrations(
      content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetRegistrationsCallback& callback
  ) override;
  void Unregister(
      content::BackgroundSyncPeriodicity periodicity, int64_t id, const mojo::String& tag, int64_t service_worker_registration_id, const UnregisterCallback& callback
  ) override;
  void GetPermissionStatus(
      content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetPermissionStatusCallback& callback
  ) override;
};


class BackgoundSyncServiceClientProxy
    : public BackgoundSyncServiceClient,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit BackgoundSyncServiceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void Sync(
      content::SyncRegistrationPtr event
  ) override;
};

class BackgroundSyncServiceStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  BackgroundSyncServiceStub();
  ~BackgroundSyncServiceStub() override;
  void set_sink(BackgroundSyncService* sink) { sink_ = sink; }
  BackgroundSyncService* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  BackgroundSyncService* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class BackgoundSyncServiceClientStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  BackgoundSyncServiceClientStub();
  ~BackgoundSyncServiceClientStub() override;
  void set_sink(BackgoundSyncServiceClient* sink) { sink_ = sink; }
  BackgoundSyncServiceClient* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  BackgoundSyncServiceClient* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class BackgroundSyncServiceRequestValidator : public mojo::MessageFilter {
 public:
  explicit BackgroundSyncServiceRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class BackgoundSyncServiceClientRequestValidator : public mojo::MessageFilter {
 public:
  explicit BackgoundSyncServiceClientRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class BackgroundSyncServiceResponseValidator : public mojo::MessageFilter {
 public:
  explicit BackgroundSyncServiceResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};




}  // namespace content

#endif  // CONTENT_COMMON_BACKGROUND_SYNC_SERVICE_MOJOM_H_
