// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_IMAGE_DOWNLOADER_IMAGE_DOWNLOADER_MOJOM_H_
#define CONTENT_COMMON_IMAGE_DOWNLOADER_IMAGE_DOWNLOADER_MOJOM_H_

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_impl.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "content/common/image_downloader/image_downloader.mojom-internal.h"
#include "skia/public/interfaces/bitmap.mojom.h"
#include "ui/mojo/geometry/geometry.mojom.h"
namespace image_downloader {

class ImageDownloader;
using ImageDownloaderPtr = mojo::InterfacePtr<ImageDownloader>;

class DownloadRequest;

using DownloadRequestPtr = mojo::InlinedStructPtr<DownloadRequest>;


class DownloadResult;

using DownloadResultPtr = mojo::StructPtr<DownloadResult>;



class ImageDownloaderProxy;
class ImageDownloaderStub;

class ImageDownloaderRequestValidator;
class ImageDownloaderResponseValidator;

class ImageDownloader {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = ImageDownloaderProxy;
  using Stub_ = ImageDownloaderStub;

  using RequestValidator_ = ImageDownloaderRequestValidator;
  using ResponseValidator_ = ImageDownloaderResponseValidator;
  virtual ~ImageDownloader() {}

  using DownloadImageCallback = mojo::Callback<void(DownloadResultPtr)>;
  virtual void DownloadImage(DownloadRequestPtr request, const DownloadImageCallback& callback) = 0;
};


class ImageDownloaderProxy
    : public ImageDownloader,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit ImageDownloaderProxy(mojo::MessageReceiverWithResponder* receiver);
  void DownloadImage(
      DownloadRequestPtr request, const DownloadImageCallback& callback
  ) override;
};

class ImageDownloaderStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  ImageDownloaderStub();
  ~ImageDownloaderStub() override;
  void set_sink(ImageDownloader* sink) { sink_ = sink; }
  ImageDownloader* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  ImageDownloader* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class ImageDownloaderRequestValidator : public mojo::MessageFilter {
 public:
  explicit ImageDownloaderRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class ImageDownloaderResponseValidator : public mojo::MessageFilter {
 public:
  explicit ImageDownloaderResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};





class DownloadRequest {
 public:
  using Data_ = internal::DownloadRequest_Data;

  static DownloadRequestPtr New();

  template <typename U>
  static DownloadRequestPtr From(const U& u) {
    return mojo::TypeConverter<DownloadRequestPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DownloadRequest>::Convert(*this);
  }

  DownloadRequest();
  ~DownloadRequest();


  DownloadRequestPtr Clone() const;
  bool Equals(const DownloadRequest& other) const;

  mojo::String url;
  bool is_favicon;
  uint32_t max_bitmap_size;
  bool bypass_cache;
};







class DownloadResult {
 public:
  using Data_ = internal::DownloadResult_Data;

  static DownloadResultPtr New();

  template <typename U>
  static DownloadResultPtr From(const U& u) {
    return mojo::TypeConverter<DownloadResultPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DownloadResult>::Convert(*this);
  }

  DownloadResult();
  ~DownloadResult();


  DownloadResultPtr Clone() const;
  bool Equals(const DownloadResult& other) const;

  int32_t http_status_code;
  mojo::Array<skia::BitmapPtr> images;
  mojo::Array<mojo::SizePtr> original_image_sizes;
};



size_t GetSerializedSize_(const DownloadRequestPtr& input);
void Serialize_(DownloadRequestPtr input, mojo::internal::Buffer* buffer,
                internal::DownloadRequest_Data** output);
void Deserialize_(internal::DownloadRequest_Data* input,
                  DownloadRequestPtr* output);

size_t GetSerializedSize_(const DownloadResultPtr& input);
void Serialize_(DownloadResultPtr input, mojo::internal::Buffer* buffer,
                internal::DownloadResult_Data** output);
void Deserialize_(internal::DownloadResult_Data* input,
                  DownloadResultPtr* output);

}  // namespace image_downloader

#endif  // CONTENT_COMMON_IMAGE_DOWNLOADER_IMAGE_DOWNLOADER_MOJOM_H_
