// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_PERMISSION_SERVICE_MOJOM_H_
#define CONTENT_COMMON_PERMISSION_SERVICE_MOJOM_H_

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_impl.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "content/common/permission_service.mojom-internal.h"
#include "content/public/common/permission_status.mojom.h"
namespace content {

enum PermissionName {
  PERMISSION_NAME_GEOLOCATION,
  PERMISSION_NAME_NOTIFICATIONS,
  PERMISSION_NAME_PUSH_NOTIFICATIONS,
  PERMISSION_NAME_MIDI_SYSEX,
  PERMISSION_NAME_PROTECTED_MEDIA_IDENTIFIER,
};

class PermissionService;
using PermissionServicePtr = mojo::InterfacePtr<PermissionService>;


class PermissionServiceProxy;
class PermissionServiceStub;

class PermissionServiceRequestValidator;
class PermissionServiceResponseValidator;

class PermissionService {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = PermissionServiceProxy;
  using Stub_ = PermissionServiceStub;

  using RequestValidator_ = PermissionServiceRequestValidator;
  using ResponseValidator_ = PermissionServiceResponseValidator;
  virtual ~PermissionService() {}

  using HasPermissionCallback = mojo::Callback<void(content::PermissionStatus)>;
  virtual void HasPermission(PermissionName permission, const mojo::String& origin, const HasPermissionCallback& callback) = 0;

  using RequestPermissionCallback = mojo::Callback<void(content::PermissionStatus)>;
  virtual void RequestPermission(PermissionName permission, const mojo::String& origin, bool user_gesture, const RequestPermissionCallback& callback) = 0;

  using RevokePermissionCallback = mojo::Callback<void(content::PermissionStatus)>;
  virtual void RevokePermission(PermissionName permission, const mojo::String& origin, const RevokePermissionCallback& callback) = 0;

  using GetNextPermissionChangeCallback = mojo::Callback<void(content::PermissionStatus)>;
  virtual void GetNextPermissionChange(PermissionName permission, const mojo::String& origin, content::PermissionStatus last_known_status, const GetNextPermissionChangeCallback& callback) = 0;
};


class PermissionServiceProxy
    : public PermissionService,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit PermissionServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void HasPermission(
      PermissionName permission, const mojo::String& origin, const HasPermissionCallback& callback
  ) override;
  void RequestPermission(
      PermissionName permission, const mojo::String& origin, bool user_gesture, const RequestPermissionCallback& callback
  ) override;
  void RevokePermission(
      PermissionName permission, const mojo::String& origin, const RevokePermissionCallback& callback
  ) override;
  void GetNextPermissionChange(
      PermissionName permission, const mojo::String& origin, content::PermissionStatus last_known_status, const GetNextPermissionChangeCallback& callback
  ) override;
};

class PermissionServiceStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  PermissionServiceStub();
  ~PermissionServiceStub() override;
  void set_sink(PermissionService* sink) { sink_ = sink; }
  PermissionService* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  PermissionService* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class PermissionServiceRequestValidator : public mojo::MessageFilter {
 public:
  explicit PermissionServiceRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class PermissionServiceResponseValidator : public mojo::MessageFilter {
 public:
  explicit PermissionServiceResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};




}  // namespace content

#endif  // CONTENT_COMMON_PERMISSION_SERVICE_MOJOM_H_
