// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_PRESENTATION_PRESENTATION_SERVICE_MOJOM_H_
#define CONTENT_COMMON_PRESENTATION_PRESENTATION_SERVICE_MOJOM_H_

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_impl.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "content/common/presentation/presentation_service.mojom-internal.h"
namespace presentation {

enum PresentationSessionState {
  PRESENTATION_SESSION_STATE_CONNECTED,
  PRESENTATION_SESSION_STATE_DISCONNECTED,
};

enum PresentationErrorType {
  PRESENTATION_ERROR_TYPE_NO_AVAILABLE_SCREENS,
  PRESENTATION_ERROR_TYPE_SESSION_REQUEST_CANCELLED,
  PRESENTATION_ERROR_TYPE_NO_PRESENTATION_FOUND,
  PRESENTATION_ERROR_TYPE_UNKNOWN,
};

enum PresentationMessageType {
  PRESENTATION_MESSAGE_TYPE_TEXT,
  PRESENTATION_MESSAGE_TYPE_ARRAY_BUFFER,
  PRESENTATION_MESSAGE_TYPE_BLOB,
};

class PresentationService;
using PresentationServicePtr = mojo::InterfacePtr<PresentationService>;

class PresentationServiceClient;
using PresentationServiceClientPtr = mojo::InterfacePtr<PresentationServiceClient>;

class PresentationSessionInfo;

using PresentationSessionInfoPtr = mojo::InlinedStructPtr<PresentationSessionInfo>;


class PresentationError;

using PresentationErrorPtr = mojo::InlinedStructPtr<PresentationError>;


class SessionMessage;

using SessionMessagePtr = mojo::StructPtr<SessionMessage>;



class PresentationServiceProxy;
class PresentationServiceStub;

class PresentationServiceRequestValidator;
class PresentationServiceResponseValidator;

class PresentationService {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = PresentationServiceProxy;
  using Stub_ = PresentationServiceStub;

  using RequestValidator_ = PresentationServiceRequestValidator;
  using ResponseValidator_ = PresentationServiceResponseValidator;
  virtual ~PresentationService() {}

  virtual void SetDefaultPresentationURL(const mojo::String& default_presentation_url, const mojo::String& default_presentation_id) = 0;

  virtual void SetClient(PresentationServiceClientPtr client) = 0;

  virtual void ListenForScreenAvailability() = 0;

  virtual void StopListeningForScreenAvailability() = 0;

  using ListenForDefaultSessionStartCallback = mojo::Callback<void(PresentationSessionInfoPtr)>;
  virtual void ListenForDefaultSessionStart(const ListenForDefaultSessionStartCallback& callback) = 0;

  using StartSessionCallback = mojo::Callback<void(PresentationSessionInfoPtr, PresentationErrorPtr)>;
  virtual void StartSession(const mojo::String& presentation_url, const StartSessionCallback& callback) = 0;

  using JoinSessionCallback = mojo::Callback<void(PresentationSessionInfoPtr, PresentationErrorPtr)>;
  virtual void JoinSession(const mojo::String& presentation_url, const mojo::String& presentation_id, const JoinSessionCallback& callback) = 0;

  using SendSessionMessageCallback = mojo::Callback<void(bool)>;
  virtual void SendSessionMessage(SessionMessagePtr message_request, const SendSessionMessageCallback& callback) = 0;

  virtual void CloseSession(const mojo::String& presentation_url, const mojo::String& presentation_id) = 0;

  virtual void ListenForSessionStateChange() = 0;

  using ListenForSessionMessagesCallback = mojo::Callback<void(mojo::Array<SessionMessagePtr>)>;
  virtual void ListenForSessionMessages(const ListenForSessionMessagesCallback& callback) = 0;
};


class PresentationServiceClientProxy;
class PresentationServiceClientStub;

class PresentationServiceClientRequestValidator;

class PresentationServiceClient {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = PresentationServiceClientProxy;
  using Stub_ = PresentationServiceClientStub;

  using RequestValidator_ = PresentationServiceClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  virtual ~PresentationServiceClient() {}

  virtual void OnScreenAvailabilityNotSupported() = 0;

  virtual void OnScreenAvailabilityUpdated(bool available) = 0;

  virtual void OnSessionStateChanged(PresentationSessionInfoPtr sessionInfo, PresentationSessionState newState) = 0;
};


class PresentationServiceProxy
    : public PresentationService,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit PresentationServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetDefaultPresentationURL(
      const mojo::String& default_presentation_url, const mojo::String& default_presentation_id
  ) override;
  void SetClient(
      PresentationServiceClientPtr client
  ) override;
  void ListenForScreenAvailability(
      
  ) override;
  void StopListeningForScreenAvailability(
      
  ) override;
  void ListenForDefaultSessionStart(
      const ListenForDefaultSessionStartCallback& callback
  ) override;
  void StartSession(
      const mojo::String& presentation_url, const StartSessionCallback& callback
  ) override;
  void JoinSession(
      const mojo::String& presentation_url, const mojo::String& presentation_id, const JoinSessionCallback& callback
  ) override;
  void SendSessionMessage(
      SessionMessagePtr message_request, const SendSessionMessageCallback& callback
  ) override;
  void CloseSession(
      const mojo::String& presentation_url, const mojo::String& presentation_id
  ) override;
  void ListenForSessionStateChange(
      
  ) override;
  void ListenForSessionMessages(
      const ListenForSessionMessagesCallback& callback
  ) override;
};


class PresentationServiceClientProxy
    : public PresentationServiceClient,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit PresentationServiceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnScreenAvailabilityNotSupported(
      
  ) override;
  void OnScreenAvailabilityUpdated(
      bool available
  ) override;
  void OnSessionStateChanged(
      PresentationSessionInfoPtr sessionInfo, PresentationSessionState newState
  ) override;
};

class PresentationServiceStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  PresentationServiceStub();
  ~PresentationServiceStub() override;
  void set_sink(PresentationService* sink) { sink_ = sink; }
  PresentationService* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  PresentationService* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class PresentationServiceClientStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  PresentationServiceClientStub();
  ~PresentationServiceClientStub() override;
  void set_sink(PresentationServiceClient* sink) { sink_ = sink; }
  PresentationServiceClient* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  PresentationServiceClient* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class PresentationServiceRequestValidator : public mojo::MessageFilter {
 public:
  explicit PresentationServiceRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class PresentationServiceClientRequestValidator : public mojo::MessageFilter {
 public:
  explicit PresentationServiceClientRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class PresentationServiceResponseValidator : public mojo::MessageFilter {
 public:
  explicit PresentationServiceResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};





class PresentationSessionInfo {
 public:
  using Data_ = internal::PresentationSessionInfo_Data;

  static PresentationSessionInfoPtr New();

  template <typename U>
  static PresentationSessionInfoPtr From(const U& u) {
    return mojo::TypeConverter<PresentationSessionInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PresentationSessionInfo>::Convert(*this);
  }

  PresentationSessionInfo();
  ~PresentationSessionInfo();


  PresentationSessionInfoPtr Clone() const;
  bool Equals(const PresentationSessionInfo& other) const;

  mojo::String url;
  mojo::String id;
};




class PresentationError {
 public:
  using Data_ = internal::PresentationError_Data;

  static PresentationErrorPtr New();

  template <typename U>
  static PresentationErrorPtr From(const U& u) {
    return mojo::TypeConverter<PresentationErrorPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, PresentationError>::Convert(*this);
  }

  PresentationError();
  ~PresentationError();


  PresentationErrorPtr Clone() const;
  bool Equals(const PresentationError& other) const;

  PresentationErrorType error_type;
  mojo::String message;
};








class SessionMessage {
 public:
  using Data_ = internal::SessionMessage_Data;

  static SessionMessagePtr New();

  template <typename U>
  static SessionMessagePtr From(const U& u) {
    return mojo::TypeConverter<SessionMessagePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SessionMessage>::Convert(*this);
  }

  SessionMessage();
  ~SessionMessage();


  SessionMessagePtr Clone() const;
  bool Equals(const SessionMessage& other) const;

  mojo::String presentation_url;
  mojo::String presentation_id;
  PresentationMessageType type;
  mojo::String message;
  mojo::Array<uint8_t> data;
};



size_t GetSerializedSize_(const PresentationSessionInfoPtr& input);
void Serialize_(PresentationSessionInfoPtr input, mojo::internal::Buffer* buffer,
                internal::PresentationSessionInfo_Data** output);
void Deserialize_(internal::PresentationSessionInfo_Data* input,
                  PresentationSessionInfoPtr* output);

size_t GetSerializedSize_(const PresentationErrorPtr& input);
void Serialize_(PresentationErrorPtr input, mojo::internal::Buffer* buffer,
                internal::PresentationError_Data** output);
void Deserialize_(internal::PresentationError_Data* input,
                  PresentationErrorPtr* output);

size_t GetSerializedSize_(const SessionMessagePtr& input);
void Serialize_(SessionMessagePtr input, mojo::internal::Buffer* buffer,
                internal::SessionMessage_Data** output);
void Deserialize_(internal::SessionMessage_Data* input,
                  SessionMessagePtr* output);

}  // namespace presentation

#endif  // CONTENT_COMMON_PRESENTATION_PRESENTATION_SERVICE_MOJOM_H_
