// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_PORT_SERVICE_MOJOM_H_
#define CONTENT_COMMON_SERVICE_PORT_SERVICE_MOJOM_H_

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_impl.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "content/common/service_port_service.mojom-internal.h"
namespace content {

enum ServicePortConnectResult {
  SERVICE_PORT_CONNECT_RESULT_ACCEPT,
  SERVICE_PORT_CONNECT_RESULT_REJECT,
};

class ServicePortService;
using ServicePortServicePtr = mojo::InterfacePtr<ServicePortService>;

class ServicePortServiceClient;
using ServicePortServiceClientPtr = mojo::InterfacePtr<ServicePortServiceClient>;

class MojoTransferredMessagePort;

using MojoTransferredMessagePortPtr = mojo::InlinedStructPtr<MojoTransferredMessagePort>;



class ServicePortServiceProxy;
class ServicePortServiceStub;

class ServicePortServiceRequestValidator;
class ServicePortServiceResponseValidator;

class ServicePortService {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = ServicePortServiceProxy;
  using Stub_ = ServicePortServiceStub;

  using RequestValidator_ = ServicePortServiceRequestValidator;
  using ResponseValidator_ = ServicePortServiceResponseValidator;
  virtual ~ServicePortService() {}

  virtual void SetClient(ServicePortServiceClientPtr client) = 0;

  using ConnectCallback = mojo::Callback<void(ServicePortConnectResult, int32_t)>;
  virtual void Connect(const mojo::String& target_url, const mojo::String& origin, const ConnectCallback& callback) = 0;

  virtual void PostMessage(int32_t port_id, const mojo::String& message, mojo::Array<MojoTransferredMessagePortPtr> ports) = 0;

  virtual void ClosePort(int32_t port_id) = 0;
};


class ServicePortServiceClientProxy;
class ServicePortServiceClientStub;

class ServicePortServiceClientRequestValidator;

class ServicePortServiceClient {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = ServicePortServiceClientProxy;
  using Stub_ = ServicePortServiceClientStub;

  using RequestValidator_ = ServicePortServiceClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  virtual ~ServicePortServiceClient() {}

  virtual void PostMessage(int32_t port_id, const mojo::String& message, mojo::Array<MojoTransferredMessagePortPtr> ports, mojo::Array<int32_t> new_routing_ids) = 0;
};


class ServicePortServiceProxy
    : public ServicePortService,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit ServicePortServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetClient(
      ServicePortServiceClientPtr client
  ) override;
  void Connect(
      const mojo::String& target_url, const mojo::String& origin, const ConnectCallback& callback
  ) override;
  void PostMessage(
      int32_t port_id, const mojo::String& message, mojo::Array<MojoTransferredMessagePortPtr> ports
  ) override;
  void ClosePort(
      int32_t port_id
  ) override;
};


class ServicePortServiceClientProxy
    : public ServicePortServiceClient,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit ServicePortServiceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void PostMessage(
      int32_t port_id, const mojo::String& message, mojo::Array<MojoTransferredMessagePortPtr> ports, mojo::Array<int32_t> new_routing_ids
  ) override;
};

class ServicePortServiceStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  ServicePortServiceStub();
  ~ServicePortServiceStub() override;
  void set_sink(ServicePortService* sink) { sink_ = sink; }
  ServicePortService* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  ServicePortService* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class ServicePortServiceClientStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  ServicePortServiceClientStub();
  ~ServicePortServiceClientStub() override;
  void set_sink(ServicePortServiceClient* sink) { sink_ = sink; }
  ServicePortServiceClient* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  ServicePortServiceClient* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class ServicePortServiceRequestValidator : public mojo::MessageFilter {
 public:
  explicit ServicePortServiceRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class ServicePortServiceClientRequestValidator : public mojo::MessageFilter {
 public:
  explicit ServicePortServiceClientRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class ServicePortServiceResponseValidator : public mojo::MessageFilter {
 public:
  explicit ServicePortServiceResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};





class MojoTransferredMessagePort {
 public:
  using Data_ = internal::MojoTransferredMessagePort_Data;

  static MojoTransferredMessagePortPtr New();

  template <typename U>
  static MojoTransferredMessagePortPtr From(const U& u) {
    return mojo::TypeConverter<MojoTransferredMessagePortPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, MojoTransferredMessagePort>::Convert(*this);
  }

  MojoTransferredMessagePort();
  ~MojoTransferredMessagePort();


  MojoTransferredMessagePortPtr Clone() const;
  bool Equals(const MojoTransferredMessagePort& other) const;

  int32_t id;
  bool send_messages_as_values;
};





size_t GetSerializedSize_(const MojoTransferredMessagePortPtr& input);
void Serialize_(MojoTransferredMessagePortPtr input, mojo::internal::Buffer* buffer,
                internal::MojoTransferredMessagePort_Data** output);
void Deserialize_(internal::MojoTransferredMessagePort_Data* input,
                  MojoTransferredMessagePortPtr* output);

}  // namespace content

#endif  // CONTENT_COMMON_SERVICE_PORT_SERVICE_MOJOM_H_
