// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MOJO_SERVICES_NETWORK_PUBLIC_INTERFACES_HTTP_MESSAGE_MOJOM_H_
#define MOJO_SERVICES_NETWORK_PUBLIC_INTERFACES_HTTP_MESSAGE_MOJOM_H_

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_impl.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/services/network/public/interfaces/http_message.mojom-internal.h"
namespace mojo {

class HttpHeader;

using HttpHeaderPtr = mojo::InlinedStructPtr<HttpHeader>;


class HttpRequest;

using HttpRequestPtr = mojo::StructPtr<HttpRequest>;


class HttpResponse;

using HttpResponsePtr = mojo::StructPtr<HttpResponse>;






class HttpHeader {
 public:
  using Data_ = internal::HttpHeader_Data;

  static HttpHeaderPtr New();

  template <typename U>
  static HttpHeaderPtr From(const U& u) {
    return mojo::TypeConverter<HttpHeaderPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, HttpHeader>::Convert(*this);
  }

  HttpHeader();
  ~HttpHeader();


  HttpHeaderPtr Clone() const;
  bool Equals(const HttpHeader& other) const;

  mojo::String name;
  mojo::String value;
};








class HttpRequest {
 public:
  using Data_ = internal::HttpRequest_Data;

  static HttpRequestPtr New();

  template <typename U>
  static HttpRequestPtr From(const U& u) {
    return mojo::TypeConverter<HttpRequestPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, HttpRequest>::Convert(*this);
  }

  HttpRequest();
  ~HttpRequest();


  bool Equals(const HttpRequest& other) const;

  mojo::String method;
  mojo::String url;
  mojo::Array<HttpHeaderPtr> headers;
  mojo::ScopedDataPipeConsumerHandle body;
};




class HttpResponse {
 public:
  using Data_ = internal::HttpResponse_Data;

  static HttpResponsePtr New();

  template <typename U>
  static HttpResponsePtr From(const U& u) {
    return mojo::TypeConverter<HttpResponsePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, HttpResponse>::Convert(*this);
  }

  HttpResponse();
  ~HttpResponse();


  bool Equals(const HttpResponse& other) const;

  uint32_t status_code;
  mojo::Array<HttpHeaderPtr> headers;
  mojo::ScopedDataPipeConsumerHandle body;
};



size_t GetSerializedSize_(const HttpHeaderPtr& input);
void Serialize_(HttpHeaderPtr input, mojo::internal::Buffer* buffer,
                internal::HttpHeader_Data** output);
void Deserialize_(internal::HttpHeader_Data* input,
                  HttpHeaderPtr* output);

size_t GetSerializedSize_(const HttpRequestPtr& input);
void Serialize_(HttpRequestPtr input, mojo::internal::Buffer* buffer,
                internal::HttpRequest_Data** output);
void Deserialize_(internal::HttpRequest_Data* input,
                  HttpRequestPtr* output);

size_t GetSerializedSize_(const HttpResponsePtr& input);
void Serialize_(HttpResponsePtr input, mojo::internal::Buffer* buffer,
                internal::HttpResponse_Data** output);
void Deserialize_(internal::HttpResponse_Data* input,
                  HttpResponsePtr* output);

}  // namespace mojo

#endif  // MOJO_SERVICES_NETWORK_PUBLIC_INTERFACES_HTTP_MESSAGE_MOJOM_H_
