// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MOJO_SERVICES_NETWORK_PUBLIC_INTERFACES_URL_LOADER_MOJOM_H_
#define MOJO_SERVICES_NETWORK_PUBLIC_INTERFACES_URL_LOADER_MOJOM_H_

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_impl.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/services/network/public/interfaces/url_loader.mojom-internal.h"
#include "mojo/services/network/public/interfaces/http_message.mojom.h"
#include "mojo/services/network/public/interfaces/network_error.mojom.h"
namespace mojo {

class URLLoader;
using URLLoaderPtr = mojo::InterfacePtr<URLLoader>;

class URLRequest;

using URLRequestPtr = mojo::StructPtr<URLRequest>;


class URLResponse;

using URLResponsePtr = mojo::StructPtr<URLResponse>;


class URLLoaderStatus;

using URLLoaderStatusPtr = mojo::StructPtr<URLLoaderStatus>;



class URLLoaderProxy;
class URLLoaderStub;

class URLLoaderRequestValidator;
class URLLoaderResponseValidator;

class URLLoader {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;

  using Proxy_ = URLLoaderProxy;
  using Stub_ = URLLoaderStub;

  using RequestValidator_ = URLLoaderRequestValidator;
  using ResponseValidator_ = URLLoaderResponseValidator;
  virtual ~URLLoader() {}

  using StartCallback = mojo::Callback<void(URLResponsePtr)>;
  virtual void Start(URLRequestPtr request, const StartCallback& callback) = 0;

  using FollowRedirectCallback = mojo::Callback<void(URLResponsePtr)>;
  virtual void FollowRedirect(const FollowRedirectCallback& callback) = 0;

  using QueryStatusCallback = mojo::Callback<void(URLLoaderStatusPtr)>;
  virtual void QueryStatus(const QueryStatusCallback& callback) = 0;
};


class URLLoaderProxy
    : public URLLoader,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit URLLoaderProxy(mojo::MessageReceiverWithResponder* receiver);
  void Start(
      URLRequestPtr request, const StartCallback& callback
  ) override;
  void FollowRedirect(
      const FollowRedirectCallback& callback
  ) override;
  void QueryStatus(
      const QueryStatusCallback& callback
  ) override;
};

class URLLoaderStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  URLLoaderStub();
  ~URLLoaderStub() override;
  void set_sink(URLLoader* sink) { sink_ = sink; }
  URLLoader* sink() { return sink_; }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  URLLoader* sink_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};

class URLLoaderRequestValidator : public mojo::MessageFilter {
 public:
  explicit URLLoaderRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};

class URLLoaderResponseValidator : public mojo::MessageFilter {
 public:
  explicit URLLoaderResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};









class URLRequest {
 public:
  using Data_ = internal::URLRequest_Data;

  static URLRequestPtr New();

  template <typename U>
  static URLRequestPtr From(const U& u) {
    return mojo::TypeConverter<URLRequestPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, URLRequest>::Convert(*this);
  }

  URLRequest();
  ~URLRequest();


  bool Equals(const URLRequest& other) const;

  mojo::String url;
  mojo::String method;
  mojo::Array<mojo::HttpHeaderPtr> headers;
  mojo::Array<mojo::ScopedDataPipeConsumerHandle> body;
  uint32_t response_body_buffer_size;
  bool auto_follow_redirects;
  bool bypass_cache;
};




class URLResponse {
 public:
  using Data_ = internal::URLResponse_Data;

  static URLResponsePtr New();

  template <typename U>
  static URLResponsePtr From(const U& u) {
    return mojo::TypeConverter<URLResponsePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, URLResponse>::Convert(*this);
  }

  URLResponse();
  ~URLResponse();


  bool Equals(const URLResponse& other) const;

  mojo::NetworkErrorPtr error;
  mojo::ScopedDataPipeConsumerHandle body;
  mojo::String url;
  mojo::String site;
  uint32_t status_code;
  mojo::String status_line;
  mojo::Array<mojo::HttpHeaderPtr> headers;
  mojo::String mime_type;
  mojo::String charset;
  mojo::String redirect_method;
  mojo::String redirect_url;
  mojo::String redirect_referrer;
};




class URLLoaderStatus {
 public:
  using Data_ = internal::URLLoaderStatus_Data;

  static URLLoaderStatusPtr New();

  template <typename U>
  static URLLoaderStatusPtr From(const U& u) {
    return mojo::TypeConverter<URLLoaderStatusPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, URLLoaderStatus>::Convert(*this);
  }

  URLLoaderStatus();
  ~URLLoaderStatus();


  URLLoaderStatusPtr Clone() const;
  bool Equals(const URLLoaderStatus& other) const;

  mojo::NetworkErrorPtr error;
  bool is_loading;
};



size_t GetSerializedSize_(const URLRequestPtr& input);
void Serialize_(URLRequestPtr input, mojo::internal::Buffer* buffer,
                internal::URLRequest_Data** output);
void Deserialize_(internal::URLRequest_Data* input,
                  URLRequestPtr* output);

size_t GetSerializedSize_(const URLResponsePtr& input);
void Serialize_(URLResponsePtr input, mojo::internal::Buffer* buffer,
                internal::URLResponse_Data** output);
void Deserialize_(internal::URLResponse_Data* input,
                  URLResponsePtr* output);

size_t GetSerializedSize_(const URLLoaderStatusPtr& input);
void Serialize_(URLLoaderStatusPtr input, mojo::internal::Buffer* buffer,
                internal::URLLoaderStatus_Data** output);
void Deserialize_(internal::URLLoaderStatus_Data* input,
                  URLLoaderStatusPtr* output);

}  // namespace mojo

#endif  // MOJO_SERVICES_NETWORK_PUBLIC_INTERFACES_URL_LOADER_MOJOM_H_
