/*
 * Copyright (C) 2011 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "platform/text/TextRun.h"

#include "platform/fonts/Character.h"

namespace blink {

struct ExpectedTextRunSize {
    const void* pointer;
    int integers[2];
    float float1;
    float float2;
    float float3;
    uint32_t bitfields : 10;
    bool b;
    RefPtr<TextRun::RenderingContext> renderingContext;
    TabSize tabSize;
};

static_assert(sizeof(TextRun) == sizeof(ExpectedTextRunSize), "TextRun should have expected size");

void TextRun::setText(const String& string)
{
    m_len = string.length();
    if (!m_len) {
        m_data.characters8 = 0;
        m_is8Bit = true;
        return;
    }
    m_is8Bit = string.is8Bit();
    if (m_is8Bit)
        m_data.characters8 = string.characters8();
    else
        m_data.characters16 = string.characters16();
}

#if ENABLE(ASSERT)
void TextRun::setCodePath(TextCodePath codePath)
{
    // ASSERT that the string does not contain any codepoints requiring the
    // complex text path when set to ForceSimple.
    ASSERT(codePath != ForceSimple || is8Bit() || Character::
        characterRangeCodePath(characters16(), length()) == SimplePath);
    m_codePath = codePath;
}
#endif // ENABLE(ASSERT)

}
